class LocalRatingsCalculator
{

    constructor()
    {
        this.ratingDatabase = {};
        this.historyDatabase = {};
        this.scannedReplays = {};
        this.weights = new LocalRatingsWeights();
    }

    buildFromScratch()
    {
        return this.buildUpdating({}, {}, {});
    }

    buildUpdating(ratingDatabase, historyDatabase, scannedReplays)
    {
        // Loop over replays
        Engine.ProfileStart("LocalRatingsUpdatingRatingDatabase");
        // Start the loop
        for (let replay of Engine.GetReplays().filter(x => !(x.directory in scannedReplays)))
        {
            let replayObj = new LocalRatingsReplay(replay);
            this.addReplay(replayObj, ratingDatabase, historyDatabase, scannedReplays);
        }
        // Now that computation has finished, we can overwrite
        this.ratingDatabase = ratingDatabase;
        this.historyDatabase = historyDatabase;
        this.scannedReplays = scannedReplays;
        Engine.ProfileStop();
    }

    addReplay(replayObj, ratingDatabase, historyDatabase, scannedReplays)
    {
        // Skip replay if invalid
        if (!replayObj.isValid)
            return;
        // Filter replay
        const filter = new LocalRatingsFilter(replayObj);
        if (filter.applies())
            return;
        // Compute individual ratings
        const players = replayObj.players;
        const scores = players.map(x => this.getScore(replayObj, x));
        const averageScore = this.getAverageScore(scores);
        const ratings = players.map((x, i) => this.getRating(scores[i], averageScore));
        // Update ratings and history database
        players.map((x, i) => this.updatePlayerRatings(x, ratings[i], ratingDatabase));
        players.map((x, i) => this.updatePlayerHistory(x, ratings[i], historyDatabase, replayObj));
        // Update list of scanned replays
        scannedReplays[replayObj.directory] = 0;
    }

    getScore(replayObj, player)
    {
        const playerIndex = replayObj.players.indexOf(player);
        const sequences = replayObj.sequences[playerIndex];
        const sequenceLength = Object.values(sequences)[0].length;
        // Return 0 if no data is available
        if (sequenceLength == 0)
            return 0;
        // Compute score
        let score = 0;
        for (let sequenceType of Object.keys(sequences))
        {
            let sequence = sequences[sequenceType];
            let configKey = sequenceType.toLowerCase();
            let weight = this.weights[configKey];
            if (weight > 0)
                score += sequence.reduce((pv, cv) => pv + cv, 0) * weight;
        }
        return score / sequenceLength;
    }

    getAverageScore(playersScores)
    {
        return playersScores.reduce((pv, cv) => pv + cv, 0) / playersScores.length;
    }

    getRating(playerScore, averageScore)
    {
        return (averageScore == 0) ? 0 : (playerScore - averageScore) / averageScore;
    }
    
    updatePlayerRatings(player, rating, ratingDatabase)
    {
        // Known player
        if (player in ratingDatabase)
        {
            const currentRating = ratingDatabase[player].rating;
            const currentMatches = ratingDatabase[player].matches;
            ratingDatabase[player] = {
                "rating": ((currentRating * currentMatches) + rating) / (currentMatches + 1),
                "matches": currentMatches + 1
            };            
        }
        // Unknown player
        else
        {
            ratingDatabase[player] = {
                "rating": rating,
                "matches": 1
            };
        }
    }
    
    updatePlayerHistory(player, rating, historyDatabase, replayObj)
    {
        // If player is unknown
        if (!(player in historyDatabase))
            historyDatabase[player] = {};
        // Now player is known, we add data
        historyDatabase[player][replayObj.directory] = rating;
    }
    
}
