class LocalRatingsMainPage
{

    constructor()
    {
        this.tableColumns = {};
    }
    
    // Filter players from the rating database according to the "Player Filter"
    filterPlayersFromRatingDatabase(ratingDatabase)
    {
        const filter = new LocalRatingsPlayerFilter(ratingDatabase);
        const filteredRatingDatabase = filter.apply();
        return filteredRatingDatabase;
    }

    // Store table columns from the original rating database, sorted by rank
    getColumnsFromRatingDatabase(ratingDatabase)
    {
        // Filter rating database
        const filteredRatingDatabase = this.filterPlayersFromRatingDatabase(ratingDatabase);
        // Create an array of players sorted by rank
        let sortedByRank = Object.keys(filteredRatingDatabase).map(x => [
            x,
            filteredRatingDatabase[x].rating,
            filteredRatingDatabase[x].matches
        ]);
        sortedByRank.sort((a, b) => b[1] - a[1]);
        // Convert JSON rating database into column format
        this.tableColumns = {
            "rank": sortedByRank.map((x, i) => i+1),
            "player": sortedByRank.map(x => x[0]),
            "rating": sortedByRank.map(x => (x[1] * 100).toFixed(2)),
            "matches": sortedByRank.map(x => x[2])
        };
    }

    // Populate the table with given columns
    populateTable(columns)
    {
        let selection = Engine.GetGUIObjectByName("localRatingsSelection");
        selection.selected = -1;
        selection.list_rank = columns.rank;
        selection.list_player = columns.player;
        selection.list_rating = columns.rating;
        selection.list_matches = columns.matches;
        // Change these last, otherwise crash
        selection.list = columns.player;
        selection.list_data = columns.player;
    }

    // Sort table rows according to column
    sortByColumn(selection)
    {
        const columnKey = selection.selected_column;
        const columnOrder = selection.selected_column_order;
        // Create an array of rows
        let rows = selection.list_rank.map((x, i) => [
            x,
            selection.list_player[i],
            selection.list_rating[i],
            selection.list_matches[i]
        ]);
        // Sort according to keys
        if (columnKey == "rank" || columnKey == "rating")
            rows.sort((a, b) => b[0]-a[0]);
        else if (columnKey == "matches")
            rows.sort((a, b) => b[3]-a[3]);
        else
            rows.sort((a, b) => b[1].localeCompare(a[1]));
        // Reverse according to column order
        if (columnOrder < 0)
            rows.reverse();
        // Create columns
        const columns = {
            "rank": rows.map(x => x[0]),
            "player": rows.map(x => x[1]),
            "rating": rows.map(x => x[2]),
            "matches": rows.map(x => x[3])
        };
        // Push to GUI
        this.populateTable(columns);
    }

    // Filtering function for player names
    filterPlayer(position, text)
    {
        return this.tableColumns.player[position].toLowerCase().includes(text);
    }

    // Called from the "Rebuild List" button
    rebuild()
    {
        this.runCalculator({}, {}, {});
    }

    // Called from the "Update List" button
    update()
    {
        // Load cache files
        let cacheHandler = new LocalRatingsCache();
        let ratingsDatabase = cacheHandler.load("ratingsDatabase");
        let historyDatabase = cacheHandler.load("historyDatabase");
        let scannedReplays = cacheHandler.load("scannedReplays");
        const lastSettings = cacheHandler.load("lastSettings");
        // Check if settings have changed, possibly ignoring certain settings keys
        const ignoredKeys = ["localratings.general", "localratings.charts", "localratings.playerfilter"];
        const currentSettings = this.getCurrentSettings();
        const hasChanged = Object.keys(lastSettings).some(key => (!ignoredKeys.some(x => key.startsWith(x)) && lastSettings[key] != currentSettings[key]));
        // If settings have changed, run calculator from an empty database
        if (hasChanged)
        {
            ratingsDatabase = {};
            historyDatabase = {};
            scannedReplays = {};
        }
        // Run calculator
        this.runCalculator(ratingsDatabase, historyDatabase, scannedReplays);
    }

    // Main function for rating calculation
    runCalculator(ratingsDatabase, historyDatabase, scannedReplays)
    {
        // Load calculator
        let calculator = new LocalRatingsCalculator();
        calculator.buildUpdating(ratingsDatabase, historyDatabase, scannedReplays);
        // Save cache
        let cacheHandler = new LocalRatingsCache();
        cacheHandler.save("ratingsDatabase", calculator.ratingDatabase);
        cacheHandler.save("historyDatabase", calculator.historyDatabase);
        cacheHandler.save("scannedReplays", calculator.scannedReplays);
        cacheHandler.save("lastSettings", this.getCurrentSettings());
        // Update table columns
        this.getColumnsFromRatingDatabase(calculator.ratingDatabase);
        // Update table according to search input
        const searchInput = Engine.GetGUIObjectByName("searchInput");
        this.searchPlayer(searchInput);
    }
    
    // Called when search input is edited, filtering table rows
    searchPlayer(searchInput)
    {
        const text = searchInput.caption.toLowerCase();
        const columns = {
            "rank": this.tableColumns.rank.filter((x, i) => this.filterPlayer(i, text)),
            "player": this.tableColumns.player.filter((x, i) => this.filterPlayer(i, text)),
            "rating": this.tableColumns.rating.filter((x, i) => this.filterPlayer(i, text)),
            "matches": this.tableColumns.matches.filter((x, i) => this.filterPlayer(i, text))
        };
        // Push to GUI
        this.populateTable(columns);
    }
    
    // Return a dictionary of the form { configkey: defaultvalue }
    getCurrentSettings()
    {
        const optionsJSON = Engine.ReadJSONFile("gui/localratings/options/options.json")
        let settings = {};
        optionsJSON.forEach(category => category.options.forEach(option => settings[option.config] = Engine.ConfigDB_GetValue("user", option.config)));
        return settings;
    }

    // Called after the first run dialog has positive response
    firstRunDialogConfirmed()
    {
        Engine.GetGUIObjectByName("firstRunDialog").hidden = true;
        this.update();
    }

    // Called when double-clicking on a table item
    showPlayerHistory()
    {
        // Get player name
        const selection = Engine.GetGUIObjectByName("localRatingsSelection");
        if (selection.selected == -1)
	    return;
        const player = selection.list_player[selection.selected];
        // Load cache file
        let cacheHandler = new LocalRatingsCache();
        const historyDatabase = cacheHandler.load("historyDatabase");
        // Retrive player's history
        const historyObject = historyDatabase[player];
        const historyRatings = Object.keys(historyObject).sort().map(x => historyObject[x]);
        let sumHistoryRatings = [historyRatings[0]];
        for (let i=1; i<historyRatings.length; i++)
            sumHistoryRatings.push(sumHistoryRatings[i-1] + historyRatings[i]);
        const historyAverageRatings = sumHistoryRatings.map((x, i) => x / (i+1));
        // Push to GUI
        Engine.PushGuiPage("page_localratings_history.xml", { "name": player, "historyRatings": historyRatings, "historyAverageRatings": historyAverageRatings });
    }

}
